#!/usr/bin/perl -w

use strict;

# This is the version in configure.ac:
my $official_version = $ARGV[0];

my $state = "unknown";
my $last_change = `darcs changes --last 1`;
if ($last_change =~ /tagged (\S+)$/ && $1 eq $official_version) {
  my $t = $1;
  if ($t =~ /pre(\d+)/) {
    $state = "prerelease $1";
  } elsif ($t =~ /rc(\d+)/) {
    $state = "release candidate $1";
  } elsif ($1 =~ /^[0-9\.]+$/) {
    $state = "release";
  } else {
    $state = "tag $t";
  }
} else {
  my $lastrepo = $ENV{PWD};
  if (-f "_darcs/prefs/defaultrepo") {
    $lastrepo = `cat _darcs/prefs/repos`;
  }
  if ($lastrepo =~ /repos\/unstable/ || $lastrepo =~ /darcs-unstable/) {
    $state = "unstable branch";
  } elsif ($lastrepo =~ /repos\/stable/ || $lastrepo =~ /darcs-stable/) {
    $state = "stable branch";
  }
}

print "$official_version ($state)\n";

if (open(FIN,"src/ThisVersion.lhs.in") && open(FOUT,">src/ThisVersion.lhs.tmp")) {
  while (<FIN>) {
    s/\@DARCS_VERSION_STATE\@/$state/g;
    s/\@DARCS_VERSION\@/$official_version/g;
    print FOUT $_;
  }
  close(FIN);
  close(FOUT);
}

my $replace = 1;
if (open(FIN,"src/ThisVersion.lhs") && open(FTMP,"src/ThisVersion.lhs.tmp")) {
  my ($old, $new);
  read FIN, $old, 10000;
  read FTMP, $new, 10000;
  $replace = $old ne $new;
  close(FIN);
  close(FTMP);
}

if ($replace) {
  rename "src/ThisVersion.lhs.tmp", "src/ThisVersion.lhs";
} else {
  unlink "src/ThisVersion.lhs.tmp";
}

